<?php

namespace Drupal\rte_mis_student_tracking\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\file\FileInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an file upload form to import students for student tracking.
 */
class BulkImportStudentsTrackingForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * Constructs the service objects.
   *
   * Class constructor.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ModuleExtensionList $extension_list_module, FileUrlGeneratorInterface $file_url_generator) {
    $this->entityTypeManager = $entityTypeManager;
    $this->moduleExtensionList = $extension_list_module;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('extension.list.module'),
      $container->get('file_url_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bulk_import_student_tracking_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Generate URL for sample csv.
    $module_path = $this->moduleExtensionList->getPath('rte_mis_student_tracking');
    $sample_path = $module_path . '/assets/students_import.xlsx';
    $uri = $this->fileUrlGenerator->generateAbsoluteString($sample_path);
    $real_path = Url::fromUri($uri, ['https' => TRUE]);

    $form['academic_session'] = [
      '#type' => 'item',
      '#title' => $this->t('Academic Session: @session', [
        '@session' => _rte_mis_core_get_previous_academic_year(),
      ]),
    ];

    $form['file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Upload file'),
      '#upload_validators' => [
        'file_validate_extensions' => ['csv xls xlsx'],
        'file_validate_size' => [5000000],
      ],
      '#upload_location' => 'public://bulk-import/student-tracking/',
      '#multiple' => FALSE,
      '#required' => TRUE,
      '#description' => $this->t('Download the <b><a href="@link">Template</a></b> file. Max 5 MB allowed.</br>(Only .csv, .xlsx, .xls files are allowed).</br><b>Note:</b> Only first sheet will be considered for data import.', [
        '@link' => $real_path->toString(),
      ]),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $file_uploaded = $form_state->getValue(['file', 0]) ?? NULL;
    if (isset($file_uploaded)) {
      $file = $this->entityTypeManager->getStorage('file')->load($file_uploaded);
      if ($file instanceof FileInterface) {
        // Create batch.
        $batch = [
          'title'      => $this->t('Importing students.'),
          'operations' => [
          [
            '\Drupal\rte_mis_student_tracking\Batch\StudentTrackingBatch::import',
            [$file_uploaded],
          ],
          ],
          'progressive' => TRUE,
          'progress_message' => '@percentage% complete. Time elapsed: @elapsed, estimated time remaining: @estimate.',
          'finished' => '\Drupal\rte_mis_student_tracking\Batch\StudentTrackingBatch::finishedCallback',
        ];

        batch_set($batch);
      }
    }
  }

}
